SystemOrganization addCategory: #ControlTests!

TestCase subclass: #BlockTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ControlTests'!

!BlockTests methodsFor: 'testing - composing' stamp: 'fbs 6/22/2011 07:40'!
testBlocksCanCompose
	| f g fg gf |
	f := [:x | x - 1].
	g := [:x | x * 2].
	fg := f compose: g. "f x = (x * 2) - 1"
	self assert: (fg value: 2) = 3.
	
	gf := g compose: f. "gf = (x - 1) * 2"
	self assert: (gf value: 1) = 0.! !

!BlockTests methodsFor: 'testing - composing' stamp: 'fbs 6/22/2011 17:37'!
testBlocksCanComposeWithSelectors
	"(1 + (2 squared))"
	self assert: (([:x | 1 + x] compose: #squared) value: 2) = 5.! !

!BlockTests methodsFor: 'testing - composing' stamp: 'fbs 6/22/2011 07:40'!
testCompositionLeftAssociates
	| f g fg gf |
	f := [:x | x - 1].
	g := [:x | x * 2].
	fg := f compose: g. "fg x = (x * 2) - 1"
	self assert: (fg value: 1) = 1.
	
	gf := g compose: f. "gf = (x - 1) * 2"
	self assert: (gf value: 1) = 0.! !

TestCase subclass: #ControlTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ControlTests'!

!ControlTests commentStamp: 'fbs 3/31/2011 17:34' prior: 0!
This class exercises the shift/reset control operators.!

!ControlTests methodsFor: 'testing - composing' stamp: 'fbs 6/22/2011 17:36'!
testBlocksCanComposeWithContinuations
	| cc |
	[1 + [:k | cc := k.] shift] reset.

	"(1 + 2) * 2 = (* (+ 1 2) 2)"
	self assert: (([:x | x * 2] compose: cc) value: 2) = 6.! !

!ControlTests methodsFor: 'testing - composing' stamp: 'fbs 6/22/2011 17:34'!
testContinuationsCanComposeWithBlocks
	| cc |
	[1 + [:k | cc := k.] shift] reset.

	self assert: ((cc compose: [:x | x * 2]) value: 2) = 5.! !

!ControlTests methodsFor: 'testing - composing' stamp: 'fbs 6/22/2011 17:34'!
testContinuationsCanComposeWithSelectors
	| cc |
	[1 + [:k | cc := k.] shift] reset.

	"(1 + (2 squared))"
	self assert: ((cc compose: #squared) value: 2) = 5.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 4/15/2011 16:42'!
testContinuationsCompose
	| add1 add2 toString |
	[ 1 + [ :k |
			add1 := k.
			k value: 0 ] shift ] reset.

	[ 2 + [ :k |
		add2 := k.
		k value: 0 ] shift ] reset.

	self assert: (add1 value: (add2 value: 0)) = 3.
	
	"shift -> [ :x | x printString ]"
	[ [ :k | toString := k. k value: 0 ] shift printString ] reset.

	self assert: (toString value: (add1 value: 0)) = '1'.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 6/20/2011 20:56'!
testEmptyStackShiftResetReturnsValue
	self assert: 1 = [ [:k | k value: 1] shift] reset.! !

!ControlTests methodsFor: 'testing - exceptions' stamp: 'fbs 4/14/2011 17:24'!
testExceptionWithinShiftEscapesContinuation
	| fun returnValue |
	fun := [ Exception signal: [ :k | k value: '2' ] shift ].
	self
		should: [ fun reset ]
		raise: Exception
		description: 'Reset ate the thrown exception'.
		
	[ fun reset ]
		on: Exception do: [ :e | returnValue := e messageText ].
	self assert: returnValue = '2' description: 'k returned wrong value'.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 4/14/2011 17:37'!
testNestedResetsDoNothing
	| fun |
	fun := [ [ 2 ] reset ].
	self assert: fun value = fun reset.
	self assert: fun reset = 2.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 6/25/2012 09:49'!
testNestedShiftResetsNest
	| innerCont outerCont |
	[1 + [ :k |
		outerCont := k. "stack reified as [ :x | 1 + x ]"
		k value: [ 2 + [ :kk |
				innerCont := kk. "stack reified as [ :x | 2 + x ]"
				kk value: 0 ] shift ] reset ] shift ] reset.

	self assert: 3 equals: (innerCont value: 1).
	self assert: 2 equals: (outerCont value: 1).
	self assert: 5 equals: ((outerCont compose: [:x | x * 2]) value: 2)! !

!ControlTests methodsFor: 'testing - exceptions' stamp: 'fbs 6/26/2012 15:04'!
testOnDoActsLikeDynamicVariable
	"The shift cuts out the setting to 2."
	[[[[:k |
		self assert: 1 equals: (DelimDynVarTestNotification signal ifNil: [3]).
		k value: 1] shift	]
			on: DelimDynVarTestNotification do: [:e | e resume: 2]] reset]
				on: DelimDynVarTestNotification do: [:e | e resume: 1]! !

!ControlTests methodsFor: 'testing - exceptions' stamp: 'fbs 6/26/2012 15:06'!
testOnDoActsLikeDynamicVariableWhenShiftCapturesChanges
	[[[[:k |
		self assert: 1 equals: (DelimDynVarTestNotification signal ifNil: [3]).
		k value: 1] shift	]
			on: DelimDynVarTestNotification do: [:e | e resume: 2]] reset]
				on: DelimDynVarTestNotification do: [:e | e resume: 1]! !

!ControlTests methodsFor: 'testing - exceptions' stamp: 'fbs 6/26/2012 15:06'!
testOnDoActsLikeDynamicVariableWhenShiftCapturesNoChanges
	[[[[:k |
		self assert: 2 equals: (DelimDynVarTestNotification signal ifNil: [3]).
		k value: 1] shift	] reset]
			on: DelimDynVarTestNotification do: [:e | e resume: 2]]
				on: DelimDynVarTestNotification do: [:e | e resume: 1]! !

!ControlTests methodsFor: 'testing - unwinds' stamp: 'fbs 4/18/2011 22:28'!
testRepeatedInvocationsShouldAlwaysRunEnsures
	| cont count |
	"[:x | 1 + ( [ x ] ensure: [ count := count + 1 ] ) ]"
	count := 0.
	[ 2 + ( [ [ :k | cont := k. k value: 0] shift ] ensure: [ count := count + 1 ] ) ] reset.
	self assert: count = 1 description: 'Ensure didn''t run the first time'.
	
	self assert: (cont value: 1) = 3.
	self assert: count = 2 description: 'Ensure didn''t run on 2nd invocation'.
	
	self assert: (cont value: 3) = 5.
	self assert: count = 3 description: 'Ensure didn''t run on 3rd invocation'.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 3/31/2011 17:36'!
testResetDoesNothingOnItsOwn
	| fun |
	fun := [1 + 2].
	self assert: fun value = fun reset.! !

!ControlTests methodsFor: 'testing - exceptions' stamp: 'fbs 4/14/2011 16:41'!
testResetDoesntEatExceptions
	self
		should: [ [ Exception signal ] reset ]
		raise: Exception
		description: 'Reset ate the thrown exception'.! !

!ControlTests methodsFor: 'testing - monad' stamp: 'fbs 6/23/2012 20:29'!
testReturnInjectsObjectIntoContinuationMonad
	| cc |
	cc := PartialContinuation return: [:x | x + 1].
	self assert: 2 equals: (cc value: 1).
	self should: [cc value: nil] raise: MessageNotUnderstood.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 4/14/2011 17:51'!
testShiftCutsToNearestReset
	| cont |
	[1 + [ 2 + [ :k |
			cont := k.
			k value: 0 ] shift ] reset ] reset.
	self assert: (cont value: 0) = 2.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 4/14/2011 17:27'!
testShiftMakesReentrantContinuation
	| cont |
	self assert: [ 2 + [ :k | cont := k. k value: 0 ] shift ] reset = 2.
	
	self assert: (cont value: 2) = 4.
	self assert: (cont value: 4) = 6.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 6/28/2011 07:49'!
testShiftPreservesResetMarker
	| cc result |
	[1 + [:k | cc := k. k value: 2] shift] reset.
	self assert: (cc stack last method selector ~= #reset) description: 'Partial continuation cut too far, eating the reset'.
	self assert: (cc stack last method selector ~= #on:do:) description: 'Partial continuation cut too far, eating the marker context'.

	"From Chung-chieh Shan's 'Shift to control': (reset (cons 'a (reset (shift f (shift g '())))))"
	result := [ 'a' , [[:f | [:g| g value: 'b'] shift ] shift] reset ] reset.
	self assert: 'ab' = result.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 4/14/2011 17:57'!
testShiftReifiesStackAsFunction
	"Stack reified as [:x | 2 + x] so the expression reduces to
		(1 + (3 + [:x | 2 + x] value: 4))
		== (1 + (3 + (2 + 4)))
		== 10."
	self assert: (1 + [ 2 + [ :k | 3 + (k value: 4) ] shift ] reset) = 10.
	
	"Stack again reified as [:x | 2 + x] so the expression reduces to
		(1 + (3 + [:x | 2 + x] value: 5 + [:x | 2 + x] value: 1))
		== (1 + (3 + (2 + 5) + (2 + 1)))
		== 14."
	self assert: (1 + [ 2 + [ :k | 3 + (k value: 5) + (k value: 1) ] shift ] reset) = 14.! !

!ControlTests methodsFor: 'testing - exceptions' stamp: 'fbs 4/15/2011 13:51'!
testShiftWithCaughtException
	| fun |
	fun := [1 + [:k | k value: ([ 2 / 0 ] on: ZeroDivide do: [ 1 ]) ] shift ].
	self shouldnt: [fun reset] raise: ZeroDivide.! !

!ControlTests methodsFor: 'testing - unwinds' stamp: 'fbs 4/18/2011 22:12'!
testShiftWithExceptionAndInnerEnsureRaisesExceptionAndRunsEnsure
	| flag fun |
	flag := false.
	fun := [ 2 + [ :k | [ 1 / 0 ] ensure: [ flag := true ] ] shift ].
	self should: [ fun reset ] raise: ZeroDivide.
	self assert: flag description: 'Ensure block didn''t run'.! !

!ControlTests methodsFor: 'testing - unwinds' stamp: 'fbs 4/15/2011 13:11'!
testShiftWithInnerEnsure
	| flag |
	flag := false.
	[ 2 + [ :k | [ 0 ] ensure: [ flag := true ] ] shift ] reset.
	self assert: flag description: 'Ensure block didn''t run'.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 4/14/2011 15:20'!
testShiftWithNoResetRaisesException
	self
		should: [[ :k | 1 + 2] shift]
		raise: MissingResetException
		description: 'No missing reset exception for BlockClosure>>shift'.! !

!ControlTests methodsFor: 'testing - unwinds' stamp: 'fbs 4/15/2011 13:47'!
testShiftWithOuterEnsure
	| cont flag |

	[ [ 2 + [ :k |
			cont := k.
			k value: 0 ] shift ] reset ] ensure: [ flag := true ].
	self assert: flag description: 'Ensure block didn''t run'.
	
	flag := false.
	cont value: 0.
	self deny: flag description: 'Ensure block ran again!!'.! !

!ControlTests methodsFor: 'testing' stamp: 'fbs 3/31/2011 17:54'!
testUnusedShiftThrowsAwayCallStack
	self assert: 1 + [2 + [:k | 3] shift] reset = 4.! !

TestCase subclass: #DelimitedDynamicVariableSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ControlTests'!

!DelimitedDynamicVariableSetTest methodsFor: 'testing - controlling' stamp: 'fbs 8/7/2012 09:31'!
testDletAllowsSettingOfManyVariables
	| d p q |
	d := DelimitedDynamicVariable default: 0.
	p := DelimitedDynamicVariable default: 0.
	q := DelimitedDynamicVariable default: 0.
	d, p, q dlet: #(1 2 3) in: [
		self assert: 1 equals: d dref.
		self assert: 2 equals: p dref.
		self assert: 3 equals: q dref.
	]! !

!DelimitedDynamicVariableSetTest methodsFor: 'testing - controlling' stamp: 'fbs 7/27/2012 09:29'!
testDletCanNest
	| d p |
	d := DelimitedDynamicVariable default: 0.
	p := DelimitedDynamicVariable default: 0.
	d, p dlet: #(1 2) in: [
		d, p dlet: #(3 4) in: [
		self assert: 3 equals: d dref.
		self assert: 4 equals: p dref]]! !

!DelimitedDynamicVariableSetTest methodsFor: 'testing - controlling' stamp: 'fbs 7/27/2012 09:28'!
testDletSetsVariablesOnlyWithinBlockScope
	| d p |
	d := DelimitedDynamicVariable default: 0.
	p := DelimitedDynamicVariable default: 0.
	d, p dlet: #(1 2) in: [].
	self assert: 0 equals: d dref.
	self assert: 0 equals: p dref.! !

!DelimitedDynamicVariableSetTest methodsFor: 'testing - controlling' stamp: 'fbs 7/27/2012 09:29'!
testDsetCanAffectDefaultValue
	| d p |
	d := DelimitedDynamicVariable default: 0.
	p := DelimitedDynamicVariable default: 0.
	d dset: 1.
	self assert: 1 equals: d dref.! !

!DelimitedDynamicVariableSetTest methodsFor: 'testing - controlling' stamp: 'fbs 8/2/2012 18:49'!
testDsetMutatesValueAtGivenScope
	| d p v1 v2 v3 v4 |
	d := DelimitedDynamicVariable default: #uninitialized.
	p := DelimitedDynamicVariable default: #uninitialized.
	d, p dlet: #(1 1) in: [
		p dset: 5.
		v1 := p dref.
		d, p dlet: #(2 2) in: [
			v2 := p dref.
			p dset: 6.
			v3 := p dref].
		v4 := p dref].
	self assert: 5 equals: v1.
	self assert: 2 equals: v2.
	self assert: 6 equals: v3.
	self assert: 5 equals: v4.
	self assert: #uninitialized equals: p dref.! !

!DelimitedDynamicVariableSetTest methodsFor: 'testing - controlling' stamp: 'fbs 7/27/2012 09:30'!
testDsetOnlyAffectsOneVariable
	| d p |
	d := DelimitedDynamicVariable default: 0.
	p := DelimitedDynamicVariable default: 0.
	d dset: 1.
	self assert: 0 equals: p dref.! !

TestCase subclass: #DelimitedDynamicVariableTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ControlTests'!

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 6/26/2012 22:39'!
testDelimitedContinuationCanCapturePartOfDynamicEnvironment
	"A shift not capturing a change in a dynamic binding."
	| p f v1 v2 v3 v4|
	p := DelimitedDynamicVariable default: #uninitialized.
	p dlet: 1 in: [
		f := p dlet: 2 in: [
			[[:k | v1 := p dref. k] shift + p dref] reset ].
		v2 := p dref.
		v3 := f value: 100.
		v4 := p dref].
	self assert: #(2 1 101 1) equals: {v1. v2. v3. v4}.! !

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 6/26/2012 22:39'!
testDelimitedContinuationCanIgnorePartOfDynamicEnvironment
	"A shift capturing a change in a dynamic binding."
	| p f v1 v2 v3 v4 |
	p := DelimitedDynamicVariable default: #uninitialized.
	p dlet: 1 in: [
		f := [p dlet: 2 in: [
				[:k | v1 := p dref. k] shift + p dref]] reset.
		v2 := p dref.
		v3 := f value: 100.
		v4 := p dref].
	self assert: #(1 1 102 1) equals: {v1. v2. v3. v4}.! !

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 6/27/2012 20:19'!
testDletsCanNest
	| p v1 v2|
	p := DelimitedDynamicVariable default: #uninitialized.
	p dlet: 1 in: [
		v1 := p dref.
		p dlet: 2 in: [v2 := p dref]].
	self assert: 1 equals: v1.
	self assert: 2 equals: v2.
	self assert: #uninitialized equals: p dref.! !

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 8/2/2012 18:39'!
testDletsCanNestWithDynVarSets
	| p d v1 v2|
	p := DelimitedDynamicVariable default: #uninitialized.
	d := DelimitedDynamicVariable default: #uninitialized.
	d, p dlet: #(1 1) in: [
		p dlet: 2 in: [
			v1 := p dref].
		v2 := p dref].
	self assert: 2 equals: v1.
	self assert: 1 equals: v2.
	self assert: #uninitialized equals: p dref.! !

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 7/14/2012 22:19'!
testDsetMutatesValueAtGivenScope
	| p v1 v2 v3 v4 |
	p := DelimitedDynamicVariable default: #uninitialized.
	p dlet: 1 in: [
		p dset: 5.
		v1 := p dref.
		p dlet: 2 in: [
			v2 := p dref.
			p dset: 6.
			v3 := p dref].
		v4 := p dref].
	self assert: 5 equals: v1.
	self assert: 2 equals: v2.
	self assert: 6 equals: v3.
	self assert: 5 equals: v1.
	self assert: #uninitialized equals: p dref.! !

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 6/26/2012 19:34'!
testDynamicLookupUsesStackBindings
	"Shift cuts out that part of the stack that sets ControlTestDynVar to 3. Thus, within the shift, ControlTestDynVar should have the outer value, 2. We use these values because they are distinct from ControlTestDynVar's default value of 1."
	| p outer inner |
	p := DelimitedDynamicVariable default: #uninitialized.
	outer := 2.
	inner := 3.
	self
		assert: outer
		equals:
			(p dlet: outer in: [
				[p dlet: inner in: [
					[:k| p dref ] shift ]] reset])! !

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 6/26/2012 19:34'!
testDynamicVariablesCanBeSet
	| p |
	p := DelimitedDynamicVariable default: #uninitialized.
	p dlet: 99 in: [
		self assert: 99 equals: p dref].! !

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 6/27/2012 09:30'!
testDynamicVariablesDontInterfere
	| p d v1 v2 |
	p := DelimitedDynamicVariable default: 0.
	d := DelimitedDynamicVariable default: 0.
	p dlet: 1 in: [
		d dlet: 2 in: [
			v1 := p dref.
			v2 := d dref]].
	self assert: 1 equals: v1.
	self assert: 2 equals: v2.! !

!DelimitedDynamicVariableTest methodsFor: 'testing' stamp: 'fbs 6/26/2012 19:35'!
testDynamicVariablesUseDefaultValues
	| p |
	p := DelimitedDynamicVariable default: #uninitialized.
	self
		assert: p default
		equals: p dref.! !

Notification subclass: #DelimDynVarTestNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ControlTests'!
